# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of Striim.
#
# Striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that manages all main components of the
player and allows them to communicate with each other.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2007 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gobject import GObject, SIGNAL_RUN_LAST, TYPE_NONE, TYPE_OBJECT, timeout_add
from gobject import TYPE_STRING, TYPE_PYOBJECT

class Player(GObject):
	"""
	This class creates a key object that exposes signals and methods
	that control the overall operation of the radio player. Almost
	all modules and plugins will have access to an instance of this
	class.
	"""

	# Plubic signals

	__gsignals__ = {
		"destroy": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"toggle-play": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"stream-error": (SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_PYOBJECT,)),
		"stream-update": (SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_PYOBJECT,)),
		"stream-message": (SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_STRING,)),
		"station-update": (SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_STRING,)),
		"mute": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"playing": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"paused": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"stop": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"album-name-found": (SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_PYOBJECT,)),
		"no-album-name-found": (SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_PYOBJECT,)),
		"album-image-found": (SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_STRING,)),
		"no-album-image-found": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"toggle-window-visibility": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"show-popup": (SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_PYOBJECT,)),
		"streamer-is-ready": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"streamer-is-busy": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"streamer-is-stopped": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"quit": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"parser-is-ready": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"parser-is-busy": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"finished-parsing": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
		"parse-list": (SIGNAL_RUN_LAST, TYPE_NONE, (TYPE_PYOBJECT,)),
		"playlist-error": (SIGNAL_RUN_LAST, TYPE_NONE, ()),
	}

	def __init__(self, manager, playlist):
		"""
		Initialize the object.

		@param self: Reference to the Player instance.
		@type self: A Player object.

		@param manager: Reference to the instance manager.
		@type manager: An InstanceManager object.

		@param playlist: Reference to shoutcast files.
		@type playlist: A List object.
		"""
		GObject.__init__(self)
		self.__init_attributes(manager, playlist)
		self.__initialize_services()
		from gobject import idle_add
		idle_add(self.__initialize_plugins)
		self.__signal_id_1 = self.connect("parser-is-ready", self.__parser_is_ready_cb)
		self.__signal_id_2 = self.connect("parser-is-busy", self.__parser_is_busy_cb)
		self.__signal_id_3 = self.connect("finished-parsing", self.__finished_parsing_cb)
		self.__signal_id_4 = self.connect("stream-update", self.__stream_update_cb)
		self.__signal_id_5 = self.connect("streamer-is-stopped", self.__streamer_is_stopped_cb)
		self.__signal_id_6 = self.connect("no-album-image-found", self.__no_image_found_cb)
		self.__signal_id_7 = self.connect("album-name-found", self.__image_found_cb)
		self.__signal_id_8 = self.connect("streamer-is-ready", self.__streamer_is_ready_cb)
		self.__signal_id_9 = self.connect("streamer-is-busy", self.__streamer_is_busy_cb)
		self.__signal_id_10 = self.connect("stream-error", self.__stream_error_cb)
		self.__signal_id_11 = self.connect("playlist-error", self.__stream_error_cb)

	def __init_attributes(self, manager, playlist):
		"""
		Initialize data attributes.

		@param self: Reference to the Player instance.
		@type self: A Player object.
		"""
		from Global import glade_file
		self.__playlist = playlist
		from gtk.glade import XML
		self.__glade = XML(glade_file, domain="striim")
		self.__window = self.__glade.get_widget("MainWindow")
		self.__manager = manager
		self.__termination_queue = set([])
		self.__parser_is_ready = False
		self.__is_playing = False
		self.__is_ready = False
		self.__is_busy = False
		self.__title = None
		self.__time = None
		self.__station = None
		self.__album_image = None
		return

########################################################################
#
#						Getters for Properties
#
########################################################################

	def __get_glade_object(self):
		return self.__glade

	def __get_window(self):
		return self.__window

	def __get_response(self):
		from utils import response
		return response

	def __get_feeback(self):
		return self.__feedback

	def __get_is_playing(self):
		return self.__is_playing

	def __get_title(self):
		return self.__title

	def __get_station(self):
		return self.__station

	def __get_album_image(self):
		return self.__album_image

	def __get_parser_is_ready(self):
		return self.__parser_is_ready

	def __get_is_ready(self):
		return self.__is_ready

	def __get_is_busy(self):
		return self.__is_busy

	def __get_time(self):
		return self.__time


########################################################################
#
#							Public APIs
#
########################################################################

	glade = property(__get_glade_object)
	response = property(__get_response)
	window = property(__get_window)
	feedback = property(__get_feeback)
	is_playing = property(__get_is_playing)
	is_ready = property(__get_is_ready)
	is_busy = property(__get_is_busy)
	parser_is_ready = property(__get_parser_is_ready)
	title = property(__get_title)
	time = property(__get_time)
	station = property(__get_station)
	album_image = property(__get_album_image)

	def trigger(self, trigger_name):
		"""
		Activate a trigger.

		@param self: Reference to the Player instance.
		@type self: A Player object.

		@param trigger_name: The name of a trigger.
		@type trigger_name: A String object.
		"""
		self.__trigger_manager.trigger(trigger_name)
		return

	def add_trigger(self, trigger):
		"""
		Add a trigger to the trigger manager.

		@param self: Reference to the Player instance.
		@type self: A Player object.

		@param trigger: Reference to a trigger object.
		@type trigger: A Trigger object.
		"""
		self.__trigger_manager.add_trigger(trigger)
		return

	def remove_trigger(self, trigger):
		"""
		Remove a trigger from the trigger manager.

		@param self: Reference to the Player instance.
		@type self: A Player object.

		@param trigger: Reference to a trigger object.
		@type trigger: A Trigger object.
		"""
		self.__trigger_manager.remove_trigger(trigger)
		return

	def add_triggers(self, triggers):
		"""
		Add triggers to the trigger manager.

		@param self: Reference to the Player instance.
		@type self: A Player object.

		@param triggers: A list of trigger objects.
		@type triggers: A List object.
		"""
		self.__trigger_manager.add_triggers(triggers)
		return

	def remove_triggers(self, triggers):
		"""
		Remove triggers from trigger manager.

		@param self: Reference to the Player instance.
		@type self: A Player object.

		@param triggers: A list of trigger objects.
		@type triggers: A List object.
		"""
		self.__trigger_manager.remove_triggers(triggers)
		return

	def get_trigger_names(self):
		"""
		Get a list of all trigger names.

		@param self: Reference to the Player instance.
		@type self: A Player object.
		"""
		return self.__trigger_manager.get_trigger_names()

	def reshow_window(self):
		"""
		Hide then show the player window.

		@param self: Reference to the Player instance.
		@type self: A Player object.
		"""
		self.trigger("hide_player")
		self.trigger("show_player")
		return

	def parse_playlist(self, playlist):
		"""
		Parse possible shoutcast files.

		@param self: Reference to the Player instance.
		@type self: A Player object.

		@param playlist: Reference to shoutcast playlist files.
		@type playlist: A List object.
		"""
		if self.__parser_is_ready:
			self.emit("parse-list", playlist)
		else:
			self.parse_playlist(playlist)
		return

	def register_object(self):
		"""
		Register a unique identification with the text editor.

		Objects that need to perform special operations before the text editor
		quits, need to call this method when they are initially created. This
		function generates a unique identification number for objects. The
		identification number should be used as an argument to the
		unregister_termination_id method after the object has finished performing
		its special operation, so that the text editor can quit successfully.

		@param self: Reference to the player instance.
		@type self: An Player object.

		@return: A unique identification number.
		@rtype: A Float object.
		"""
		from utils import generate_random_number
		termination_id = generate_random_number(self.__termination_queue)
		self.__termination_queue.add(termination_id)
		return termination_id

	def unregister_object(self, termination_id):
		"""
		Unregister an object.

		This function disassociates an object from striim.

		@param self: Reference to the player.
		@type self: An Player object.

		@param termination_id: A unique identification number
		@type termination_id: A Float object.
		"""
		if termination_id in self.__termination_queue:
			self.__termination_queue.remove(termination_id)
		if not self.__termination_queue:
			self.__manager.quit()
		return

	def update_playlist(self, dictionary):
		"""
		Update shoutcast stream playlist.

		@param self: Reference to the Player instance.
		@type self: A Player object.

		@param dictionary: Reference to the playlist dictionary.
		@type dictionary: A Dictionary object.
		"""
		from operator import not_
		if not_(dictionary): return
		from Metadata import update_playlist_database
		update_playlist_database(dictionary)
		return

	def update_stations(self, dictionary):
		return

########################################################################
#
#							Helper Methods
#
########################################################################

	def __initialize_services(self):
		"""
		Initialize core services.

		@param self: Reference to the Player instance.
		@type self: A Player object.
		"""
		from TriggerManager import TriggerManager
		self.__trigger_manager = TriggerManager(self.__window)
		from WindowTrigger import Trigger
		Trigger(self)
		from Feedback import FeedbackManager
		self.__feedback = FeedbackManager(self)
		return False

	def __initialize_plugins(self):
		"""
		Initialize the plugin system.

		@param self: Reference to the Player instance.
		@type self: A Player object.
		"""
		from PluginManager import PluginManager
		PluginManager(self)
		return False

########################################################################
#
#						Event and Signal Handlers
#
########################################################################

	def __parser_is_ready_cb(self, manager):
		"""
		Handles callback when the "parser-is-ready" signal is emitted.

		@param self: Reference to the Player instance.
		@type self: A Player object.
		"""
		self.__parser_is_ready = True
		from operator import not_
		if not_(self.__playlist): return
		self.emit("parse-list", self.__playlist)
		self.__playlist = None
		return

	def __parser_is_busy_cb(self, manager):
		"""
		Handles callback when the "parser-is-busy" signal is emitted.

		@param self: Reference to the Player instance.
		@type self: A Player object.
		"""
		self.__parser_is_ready = False
		return

	def __finished_parsing_cb(self, manager):
		"""
		Handles callback when the "finished-parsing" signal is emitted.

		@param self: Reference to the Player instance.
		@type self: A Player object.
		"""
		self.__parser_is_ready = True
		return

	def __stream_update_cb(self, player, data):
		"""
		Handles callback when the "stream-update" signal is emitted.

		@param self: Reference to the Player instance.
		@type self: A Player object.
		"""
		self.__is_playing = True
		self.__is_busy = False
		self.__title = data["title"]
		self.__station = data["station"]
		self.__time = data["time"]
		return

	def __streamer_is_stopped_cb(self, *args):
		"""
		Handles callback when the "streamer-is-stopped" signal is emitted.

		@param self: Reference to the Player instance.
		@type self: A Player object.
		"""
		self.__reset_attributes()
		return

	def __streamer_is_ready_cb(self, *args):
		"""
		Handles callback when the "streamer-is-busy" signal is emitted.

		@param self: Reference to the Player instance.
		@type self: A Player object.
		"""
		self.__is_ready = True
		return

	def __streamer_is_busy_cb(self, *args):
		"""
		Handles callback when the "streamer-is-busy" signal is emitted.

		@param self: Reference to the Player instance.
		@type self: A Player object.
		"""
		self.__is_busy = True
		return

	def __stream_error_cb(self, *args):
		"""
		Handles callback when the "stream-error" signal is emitted.

		@param self: Reference to the Player instance.
		@type self: A Player object.
		"""
		self.__reset_attributes()
		return

	def __no_image_found_cb(self, *args):
		"""
		Handles callback when the "no-album-image-found" is emitted.

		@param self: Reference to the Player instance.
		@type self: A Player object.
		"""
		self.__album_image = None
		return

	def __image_found_cb(self, player, image):
		"""
		Handles callback when the "album-image-found" signal is emitted.

		@param self: Reference to the Player instance.
		@type self: A Player object.
		"""
		self.__album_image = image
		return

	def __reset_attributes(self):
		"""
		Reset key attributes.

		@param self: Reference to the Player instance.
		@type self: A Player object.
		"""
		self.__is_busy = False
		self.__is_playing = False
		self.__title = None
		self.__station = None
		self.__album_image = None
		return
