# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of Striim.
#
# Striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

class Window(object):
	"""
	This class creates an object that manages the player's window.
	"""

	def __init__(self, trigger, player):
		"""
		Initialize the object.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__init_attributes(trigger, player)
		self.__set_properties()
		self.__can_show_window()
		self.__sig_id_1 = player.connect("streamer-is-busy", self.__playing_cb)
		self.__sig_id_2 = player.connect("streamer-is-stopped", self.__paused_cb)
		self.__sig_id_9 = player.connect("playlist-error", self.__paused_cb)
		self.__sig_id_3 = player.connect("stream-update", self.__stream_update_cb)
		self.__sig_id_4 = player.connect("stream-error", self.__paused_cb)
		self.__sig_id_5 = player.connect("album-image-found", self.__album_image_found_cb)
		self.__sig_id_6 = player.connect("no-album-image-found", self.__no_album_image_found_cb)
		self.__sig_id_7 = trigger.connect("destroy", self.__destroy_cb)
		self.__sig_id_8 = self.__window.connect("delete-event", self.__delete_event_cb)

	def __init_attributes(self, trigger, player):
		"""
		Initialize data attributes.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		self.__player = player
		self.__trigger = trigger
		self.__is_visible = False
		self.__window = player.window
		self.__icon = self.__window.get_icon()
		self.__sig_id_1 = self.__sig_id_2 = self.__sig_id_3 = None
		self.__sig_id_4 = self.__sig_id_5 = self.__sig_id_6 = None
		self.__sig_id_7 = self.__sig_id_8 = None
		return

	def __set_properties(self):
		"""
		Set default properties.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		from operator import not_
		if not_(self.__player.is_playing): return
		from xml.sax.saxutils import unescape
		self.__window.set_property("title", unescape(self.__player.title))
		if not_(self.__player.album_image): return
		self.__set_image_icon(self.__player.album_image)
		return

########################################################################
#
#							Public API
#
########################################################################

	def close_window(self):
		"""
		Hide the window.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.__window.hide_all()
		return

	def show_window(self):
		"""
		Show window.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.__show_window()
		return

	def hide_window(self):
		"""
		Hide window.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.__hide_window()
		return

	def toggle_window(self):
		"""
		Toggle window visibility.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		if self.__is_visible:
			self.__hide_window()
		else:
			self.__show_window()
		return

########################################################################
#
#							Helper Methods
#
########################################################################

	def __position_window(self):
		"""
		Position window.

		Window position is got from the position database.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		from Metadata import get_window_position_from_database
		position = get_window_position_from_database()
		from operator import not_
		if not_(position): return
		self.__window.move(position[0], position[1])
		return

	def __show_window(self):
		"""
		Show window.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		if self.__is_visible: return
		self.__position_window()
		self.__player.response()
		self.__window.show_all()
		self.__player.response()
		self.__is_visible = True
		from Metadata import update_visibility_database
		update_visibility_database(self.__is_visible)
		return

	def __hide_window(self):
		"""
		Hide window.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		from operator import not_
		if not_(self.__is_visible): return
		from Metadata import update_window_position_in_database
		update_window_position_in_database(self.__window.get_position())
		self.__player.response()
		self.__window.hide()
		self.__player.response()
		self.__is_visible = False
		from Metadata import update_visibility_database
		update_visibility_database(self.__is_visible)
		return

	def __can_show_window(self):
		"""
		Whether or not to show the window.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		from Metadata import get_visibility_from_database
		from operator import not_
		if not_(get_visibility_from_database()): return
		self.__show_window()
		return

	def __set_default_icon(self):
		"""
		Set default striim icon on window.

		@param self: Reference to the Manager. instance.
		@type self: A Manager. object.
		"""
		self.__player.response()
		self.__window.set_icon(self.__icon)
		self.__player.response()
		return

	def __set_image_icon(self, image):
		"""
		Change default striim icon to image given.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param image: A image file.
		@type image: An String object.
		"""
		self.__player.response()
		self.__window.set_icon_from_file(image)
		self.__player.response()
		return

########################################################################
#
#						Event and Signal Handlers
#
########################################################################

	def __playing_cb(self, *args):
		"""
		Handles callback when the "playing" signal is emitted.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		from i18n import msg0001
		self.__window.set_property("title", msg0001)
		return

	def __paused_cb(self, *args):
		"""
		Handles callback when the "paused" signal is emitted.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.__window.set_property("title", "striim")
		self.__set_default_icon()
		return

	def __stream_update_cb(self, player, dictionary):
		"""
		Handles callback when the "stream-update" signal is emitted.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		from xml.sax.saxutils import unescape
		self.__window.set_property("title", unescape(dictionary["title"]))
		return

	def __no_album_image_found_cb(self, *args):
		"""
		Handles callback when the "no-album-image-found" signal is emitted.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.__set_default_icon()
		return

	def __album_image_found_cb(self, player, image):
		"""
		Handles callback when the "album-image-found" signal is emitted.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.

		@param player: Reference to the Player instance.
		@type player: A Player object.

		@param image: An image file.
		@type image: A String object.
		"""
		self.__set_image_icon(image)
		return

	def __destroy_cb(self, *args):
		"""
		Destroy window object.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		from Striim.utils import disconnect_signal, delete_attributes
		disconnect_signal(self.__sig_id_1, self.__player)
		disconnect_signal(self.__sig_id_2, self.__player)
		disconnect_signal(self.__sig_id_3, self.__player)
		disconnect_signal(self.__sig_id_4, self.__player)
		disconnect_signal(self.__sig_id_5, self.__player)
		disconnect_signal(self.__sig_id_6, self.__player)
		disconnect_signal(self.__sig_id_7, self.__trigger)
		disconnect_signal(self.__sig_id_8, self.__window)
		self.__window.destroy()
		delete_attributes(self)
		self = None
		del self
		return

	def __delete_event_cb(self, *args):
		"""
		Handles callback when the "delete-event" signal is emitted.

		@param self: Reference to the Manager instance.
		@type self: A Manager object.
		"""
		self.__player.trigger("close_player")
		return True
