# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of striim.
#
# striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that defines the behavior of the toggle
play menu item of the popup menu.

@author: Lateef Alabi-Oki
@organization: The striim Project
@copyright: Copyright © 2007 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import ImageMenuItem

class PlayMenuItem(ImageMenuItem):
	"""
	This class defines the behavior of the PlayMenuItem.
	"""

	def __init__(self, player):
		"""
		Initialize object.

		@param self: Reference to the PlayMenuItem instance.
		@type self: A PlayMenuItem object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		ImageMenuItem.__init__(self)
		self.__init_attributes(player)
		self.__set_properties()
		self.__signal_id_1 = self.connect("activate", self.__activate_cb)
		self.__signal_id_2 = player.connect("streamer-is-busy", self.__is_busy_cb)
		self.__signal_id_3 = player.connect("stream-error", self.__error_cb)
		self.__signal_id_4 = player.connect("playlist-error", self.__error_cb)
		self.__signal_id_5 = player.connect("stream-message", self.__update_cb)
		self.__signal_id_6 = player.connect("streamer-is-stopped", self.__error_cb)

	def __init_attributes(self, player):
		"""
		Initialize data attributes.

		@param self: Reference to the PlayMenuItem instance.
		@type self: A PlayMenuItem object.

		@param player: Reference to the Player instance.
		@type player: A Player object.
		"""
		from gtk import Image, Label
		self.__player = player
		self.__image = Image()
		self.__label = Label()
		self.__is_play = False
		return

	def __set_properties(self):
		"""
		Define default properties.

		@param self: Reference to the PlayMenuItem instance.
		@type self: A PlayMenuItem object.
		"""
		self.__label.set_use_underline(True)
		self.set_property("image", self.__image)
		from gtk import HBox
		hbox = HBox()
		self.add(hbox)
		hbox.pack_start(self.__label, False, False, 0)
		if self.__player.is_playing:
			self.set_property("sensitive", True)
			self.__show_pause()
		else:
			self.set_property("sensitive", False)
			self.__show_play()
		return

	def __show_play(self):
		"""
		Show play menu item.

		@param self: Reference to the PlayMenuItem instance.
		@type self: A PlayMenuItem object.
		"""
		self.__player.response()
		if self.__is_play: return
		from gtk import STOCK_MEDIA_PLAY, ICON_SIZE_MENU
		self.__image.clear()
		self.__image.set_from_stock(STOCK_MEDIA_PLAY, ICON_SIZE_MENU)
		from i18n import msg0003
		self.__label.set_label(msg0003)
		self.__is_play = True
		self.__player.response()
		return

	def __show_pause(self):
		"""
		Show the pause menu item.

		@param self: Reference to the PlayMenuItem instance.
		@type self: A PlayMenuItem object.
		"""
		self.__player.response()
		from gtk import STOCK_MEDIA_PAUSE, ICON_SIZE_MENU
		self.__image.clear()
		self.__image.set_from_stock(STOCK_MEDIA_PAUSE, ICON_SIZE_MENU)
		from i18n import msg0004
		self.__label.set_label(msg0004)
		self.__is_play = False
		self.__player.response()
		return

	def __activate_cb(self, *args):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the PlayMenuItem instance.
		@type self: A PlayMenuItem object.
		"""
		self.set_property("sensitive", False)
		self.__player.trigger("play")
		return

	def __is_busy_cb(self, *args):
		"""
		Handles callback when the "streamer-is-busy" signal is emitted.

		@param self: Reference to the PlayMenuItem instance.
		@type self: A PlayMenuItem object.
		"""
		self.set_property("sensitive", False)
		self.__show_pause()
		return

	def __error_cb(self, *args):
		"""
		Handles callback when the an error occurs of the player is stopped.

		@param self: Reference to the PlayMenuItem instance.
		@type self: A PlayMenuItem object.
		"""
		self.set_property("sensitive", True)
		self.__show_play()
		return

	def __update_cb(self, *args):
		"""
		Handles callback when the "stream-update" signal is emitted.

		@param self: Reference to the PlayMenuItem instance.
		@type self: A PlayMenuItem object.
		"""
		self.set_property("sensitive", True)
		self.__show_pause()
		return
